<?php

namespace EventPortal\Tests\Unit\Table;

use EventPortal\Domain\Component\Column\TextColumn;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class TableTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test **/
    public function can_make()
    {
        $table = Table::make('event.index')->model(new Event)->get();

        $this->assertInstanceOf(\EventPortal\Domain\Table\Table::class, $table);
    }

    /** @test **/
    public function uses_model_column_defaults_if_no_columns_specified()
    {
        factory(Event::class)->create();

        $table = Table::make('event.index')
            ->model(new Event)
            ->setColumns(function () {
                return [
                    TextColumn::make('Name')->default(),
                    TextColumn::make('Status'),
                ];
            })
            ->get();

        $this->assertCount(2, $table->columns);
    }

    /** @test **/
    public function can_specify_columns_to_use()
    {
        factory(Event::class)->create();

        $table = Table::make('event.index')
            ->setColumns(function () {
                return [
                    TextColumn::make('Name'),
                    TextColumn::make('Status'),
                ];
            })
            ->model(new Event)
            ->columns('name,status')
            ->get();

        $this->assertCount(3, $table->columns);
    }

    /** @test **/
    public function can_handle_specifing_columns_that_do_not_exist()
    {
        factory(Event::class)->create();

        $table = Table::make('event.index')
            ->model(new Event)
            ->setColumns(function () {
                return [
                    TextColumn::make('Name'),
                    TextColumn::make('Status'),
                ];
            })
            ->columns('name,status,slug')
            ->get();

        $this->assertCount(3, $table->columns);
    }

    /** @test **/
    public function filters_are_made_when_available()
    {
        factory(Event::class)->create();

        $table = Table::make('event.index')
            ->setColumns(function () {
                return [
                    TextColumn::make('Name'),
                    TextColumn::make('Status'),
                ];
            })
            ->model(new Event)
            ->columns('status')
            ->get();

        $this->assertCount(1, $table->filterFields);
    }

    /** @test **/
    public function context_is_added_to_route_params()
    {
        $event = factory(Event::class)->create();

        $table = Table::make('event.index')->model($event)->context($event)->get();

        $this->assertEquals(['event' => $event->id], $table->routeParams);
    }

    /** @test */
    public function select_extra_columns()
    {
        $form = factory(Form::class)->state('snapshot')->create();

        $table = Table::make('form.index')
            ->query(Form::where('id', $form->id)->select(['event_id']))
            ->get();

        $this->assertMatchesSnapshot(json_encode($table->data));
    }
}
